from typing import Any, Dict, List

from tenacity import (
    retry,
    retry_if_exception,
    stop_after_attempt,
    stop_after_delay,
    wait_fixed,
)

from filum_utils.clients.common import BaseClient, retry_if_error
from filum_utils.clients.knowledge_base.enums import KBDocumentStatusEnum
from filum_utils.clients.knowledge_base.types import (
    PreprocessUpsertDocumentRequestType,
    UpsertDocumentDataType,
)
from filum_utils.config import config

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_fixed(10),
    "stop": (stop_after_attempt(6) | stop_after_delay(60)),
    "retry": retry_if_exception(retry_if_error),
}


class KnowledgeBaseClient(BaseClient):
    def __init__(self, organization_id: str):
        super().__init__(
            base_url=config.KNOWLEDGE_BASE_SERVICE_URL,
            username=config.KNOWLEDGE_BASE_SERVICE_USERNAME,
            password=config.KNOWLEDGE_BASE_SERVICE_PASSWORD,
        )
        self._organization_id = organization_id

    @retry(**_RETRY_PARAMS)
    def upsert_documents(
        self, documents: List[UpsertDocumentDataType]
    ) -> Dict[str, Any]:
        return self._request(
            method="PUT",
            endpoint="/documents/from-records",
            params={"organization_id": self._organization_id},
            data={"documents": documents},
        )

    @retry(**_RETRY_PARAMS)
    def delete_document(self, document_id: str):
        return self._request(
            method="DELETE",
            endpoint=f"/documents/{document_id}",
            params={"organization_id": self._organization_id},
        )

    @retry(**_RETRY_PARAMS)
    def update_document_status(
        self, document_id: str, new_status: KBDocumentStatusEnum
    ):
        return self._request(
            method="PUT",
            endpoint=f"/documents/{document_id}",
            params={"organization_id": self._organization_id},
            data={"status": new_status},
        )

    @retry(**_RETRY_PARAMS)
    def preprocess_upsert_document(
        self, documents: List[PreprocessUpsertDocumentRequestType]
    ):
        self._request(
            method="POST",
            endpoint="/documents/preprocess-upsert-jobs",
            params={"organization_id": self._organization_id},
            data={"documents": documents},
        )



import csv
import io
import json
from typing import Any, BinaryIO, Dict, List

from google.cloud import pubsub_v1, storage

from filum_utils.clients.contact_center.types import PubsubMessageType
from filum_utils.config import config


class GoogleCloudClient:
    def __init__(self, project_id: str = config.GOOGLE_PROJECT_ID):
        self._project_id = project_id

    def publish_messages(
        self,
        messages: List[PubsubMessageType],
        topic_id: str = config.GOOGLE_PUBSUB_TOPIC_ID,
    ):
        publisher_options = pubsub_v1.types.PublisherOptions()
        publisher = pubsub_v1.PublisherClient(publisher_options=publisher_options)

        topic_path = publisher.topic_path(self._project_id, topic_id)

        for message in messages:
            message = json.dumps(message)
            data = message.encode("utf-8")
            publisher.publish(topic_path, data=data)


class GoogleCloudStorageClient:
    def __init__(
        self,
        project_id: str = config.GOOGLE_PROJECT_ID,
        bucket_name: str = config.GCP_UPLOADS_BUCKET,
    ):
        self._project_id = project_id

        self._storage_client = storage.Client(project=self._project_id)
        self._bucket = self._storage_client.bucket(bucket_name)

    def upload_csv_file(
        self, file_name: str, keys: List[str], rows: List[Dict[str, Any]]
    ):
        # Transform the list of users into CSV-formatted bytes
        csv_buffer = io.StringIO()
        csv_writer = csv.DictWriter(csv_buffer, fieldnames=keys)
        csv_writer.writeheader()

        for row in rows:
            # Use empty string for missing values
            csv_writer.writerow({prop: row.get(prop, "") for prop in keys})

        csv_bytes = csv_buffer.getvalue().encode("utf-8")

        # Upload the CSV file to Google Cloud Storage
        self._upload_file(file_name, io.BytesIO(csv_bytes))
        csv_buffer.close()

    def _upload_file(self, file_name: str, file_obj: BinaryIO):
        blob = self._bucket.blob(file_name)
        blob.upload_from_file(file_obj)



from tenacity import (
    retry_if_exception,
    stop_after_attempt,
    stop_after_delay,
    wait_fixed,
)

from filum_utils.clients.common import BaseClient, retry_if_error
from filum_utils.config import config

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_fixed(10),
    "stop": (stop_after_attempt(6) | stop_after_delay(60)),
    "retry": retry_if_exception(retry_if_error),
}


class ContactCenterClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.CONVERSATION_BASE_URL,
            username=config.CONVERSATION_USERNAME,
            password=config.CONVERSATION_PASSWORD,
        )



PROPERTY_PLACEHOLDER_REGEX = r'\{{[^{}]*\}}'
TRANSLATION_FORMAT_REGEX = r'question.[\d].option.[\d]'


class BaseEnum:
    @classmethod
    def get_list(cls):
        return [getattr(cls, attr) for attr in dir(cls) if attr.isupper()]


class ObjectName(BaseEnum):
    SEGMENT = "Segment"
    EVENT = "Event"
    CAMPAIGN = "Campaign"
    USER = "User"
    SURVEY_RESPONSE = "Response"
    CONVERSATION = "Conversation"


class VariableProperty:
    ID = "ID"
    LINK = "Link"
    USER_PHONE = "Phone"
    USER_EMAIL = "Email"
    RATING = "Rating"
    TEXT = "Text"
    USER = "User"
    NAME = "Name"
    SIZE = "Size"
    METRIC_TYPE = "Metric_Type"


class UserProperty:
    ID = "User ID"
    NAME = "Name"
    EMAIL = "Email"
    PHONE = "Phone"


class SurveyAnsweredEventProperty:
    USER_EMAIL = "User Email"
    USER_PHONE = "User Phone"
    SURVEY_RESPONSE_ID = "Survey Response ID"
    RATING = "Rating"
    METRIC_TYPE = "Metric Type"
    TEXT = "Follow Up Text"
    SURVEY_ID = "Survey ID"
    EVENT_NAME = "Event Name"
    REVIEW_KEY = "_keys"
    LOCATION_ID = "Location ID"


class ConversationProperty:
    ID = "ID"


class SurveyResponseProperty:
    ID = "ID"


class CampaignProperty:
    ID = "ID"
    NAME = "Name"


class SegmentProperty:
    ID = "id"
    NAME = "name"
    TOTAL_USERS = "total_users"



from typing import Any, Dict, List, Optional, TypedDict

from tenacity import (
    retry,
    retry_if_exception,
    stop_after_attempt,
    stop_after_delay,
    wait_fixed,
)

from filum_utils.clients.common import BaseClient, retry_if_error
from filum_utils.config import config

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_fixed(10),
    "stop": (stop_after_attempt(6) | stop_after_delay(60)),
    "retry": retry_if_exception(retry_if_error),
}


class ActionError(TypedDict, total=False):
    type: str
    message: Optional[str]
    sub_message: Optional[str]
    data: Dict[str, Any]
    notification_message: Optional[str]


class ActionsClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD,
        )

    @retry(**_RETRY_PARAMS)
    def get_actions(
        self, action_type_id: int, organization_id: str = None
    ) -> Optional[List[Dict[str, Any]]]:
        return (
            self._request(
                method="GET",
                endpoint="/internal/actions",
                params={
                    "action_type_id": action_type_id,
                    "organization_id": organization_id,
                },
            )
            or []
        )


class ActionClient(BaseClient):
    def __init__(self, action_id: int = None, action: Dict[str, Any] = None):
        super().__init__(
            base_url=config.APPSTORE_BASE_URL,
            username=config.APPSTORE_USERNAME,
            password=config.APPSTORE_PASSWORD,
        )

        if action:
            self.action = action

        else:
            self.action = self._request(
                method="GET", endpoint=f"/source/actions/{action_id}"
            )

    def get_data(self, key):
        data = self.action.get("data")
        return data and data.get(key)

    @retry(**_RETRY_PARAMS)
    def update_data(self, updated_data: Dict[str, Any]):
        self.action = self._request(
            method="PUT",
            endpoint=f"/source/actions/{self.action['id']}/data",
            data=updated_data,
        )



from typing import TypedDict, Optional, Dict, Any

from filum_utils.types.account import Account


class EngagementCampaign(TypedDict, total=False):
    id: str
    name: str
    account: Account
    data: Optional[Dict[str, Any]]
    daily_message_limit: Optional[int]
    last_no_limit_day: Optional[str]  # ISO date
    last_limit_reached_day: Optional[str]  # ISO date



from typing import Any, Dict, List, Optional, TypedDict, Union

from filum_utils.clients.contact_center.enums import (
    ContactCenterMessageTypeEnum,
    DocumentColumnDataTypeEnum,
    DocumentColumnFilterTypeEnum,
    DocumentDataFormatEnum,
    DocumentSyncStatusEnum,
    MessageSenderTypeEnum,
    MessageStatusEnum,
    PubSubMessageEventNameEnum,
)


# Conversation Types
class MessageUserType(TypedDict, total=False):
    id: str
    name: Optional[str]
    phone: Optional[str]
    email: Optional[str]
    avatar: Optional[str]


class MessageAgentType(TypedDict, total=False):
    id: str
    name: Optional[str]
    email: Optional[str]


class MessageRecordingType(TypedDict, total=False):
    url: Optional[str]
    duration_seconds: Optional[int]


class MessageAttachmentType(TypedDict, total=False):
    internal_url: Optional[str]
    url: str
    size_bytes: Optional[int]
    sizeBytes: Optional[int]
    type: str
    name: Optional[str]
    mime_type: Optional[str]
    mimeType: Optional[str]
    is_sticker: Optional[bool]


class PubsubMessageEventType(TypedDict, total=False):
    original_conversation_id: Optional[str]
    conversation_name: Optional[str]
    timestamp: Optional[str]
    user: MessageUserType
    original_message_id: str
    sender_type: MessageSenderTypeEnum
    original_agent: Optional[MessageAgentType]
    call_center_phone: Optional[str]
    text: Optional[str]
    conversation_link: Optional[str]
    recording: Optional[MessageRecordingType]
    note: Optional[str]
    hangup_cause: Optional[str]
    charge_fee: Optional[float]
    attachments: Optional[List[MessageAttachmentType]]
    campaign_id: Optional[str]
    campaign_name: Optional[str]
    sync_outbound: Optional[bool]
    reply_to_original_message_id: Optional[str]
    is_test: Optional[bool]

    post_data: Optional[Dict[str, Any]]
    scope: Optional[str]
    type: Optional[ContactCenterMessageTypeEnum]
    is_hidden_in_third_party: Optional[bool]
    is_edited: Optional[bool]


class PubsubMessageStatusEventType(TypedDict, total=False):
    timestamp: Optional[str]
    original_message_id: str
    status: MessageStatusEnum
    internal_message_id: Optional[str]


class PubsubMessageType(TypedDict, total=False):
    access_token: str
    organization_id: str
    installed_source_id: int
    source_id: str
    source: str
    event_name: PubSubMessageEventNameEnum
    data: Union[PubsubMessageEventType, PubsubMessageStatusEventType]


# Document Types
class KnowledgeBaseType(TypedDict):
    id: str
    indexName: str


class KBDocumentType(TypedDict):
    knowledgeBase: KnowledgeBaseType
    _id: str
    name: str
    status: str
    embeddingStatus: str
    accountId: int
    organizationId: str
    createdAt: str
    updatedAt: str
    installedSourceId: int
    collectionName: Optional[str]


class ColumnSchemaType(TypedDict, total=False):
    column_name: str
    data_type: DocumentColumnDataTypeEnum
    filter_type: DocumentColumnFilterTypeEnum
    unique_count: int
    unique_values: List[str]
    array_value_delimiter: Optional[str]
    is_primary_key: bool


class UpdateDocumentEmbeddingType(TypedDict, total=False):
    document_id: str
    name: str
    description: Optional[str]
    status: DocumentSyncStatusEnum
    progress: int
    error_message: Optional[str]
    data_format: DocumentDataFormatEnum
    categories: List[str]
    collection_name: str
    column_schemas: List[ColumnSchemaType]
    table_records: List[Dict[str, Any]]
    objectKey: Optional[str]



from typing import List

from tenacity import (
    retry,
    retry_if_exception,
    stop_after_attempt,
    stop_after_delay,
    wait_fixed,
)

from filum_utils.clients.common import BaseClient, retry_if_error
from filum_utils.config import config
from filum_utils.types.segment import UserCustomProperty

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_fixed(10),
    "stop": (stop_after_attempt(6) | stop_after_delay(60)),
    "retry": retry_if_exception(retry_if_error),
}


class SegmentClient(BaseClient):
    def __init__(self, organization_id: str):
        super().__init__(
            base_url=config.SEGMENT_SERVICE_URL,
            username=config.SEGMENT_SERVICE_USERNAME,
            password=config.SEGMENT_SERVICE_PASSWORD,
        )
        self.organization_id = organization_id

    @retry(**_RETRY_PARAMS)
    def get_user_custom_property(self, filum_user_id: str, custom_property_name: str):
        users = self.get_users_custom_properties(
            filum_user_ids=[filum_user_id], custom_property_names=[custom_property_name]
        )

        if not users:
            return None

        custom_properties = users[0].get("custom_properties") or {}
        return custom_properties.get(custom_property_name)

    @retry(**_RETRY_PARAMS)
    def get_users_custom_properties(
        self, filum_user_ids: List[str], custom_property_names: List[str]
    ) -> List[UserCustomProperty]:
        return (
            self._request(
                method="POST",
                endpoint="/find-users-prebuilt-custom-properties",
                params={"org_id": self.organization_id},
                data={
                    "filum_users_ids": filum_user_ids,
                    "custom_properties_names": custom_property_names,
                },
            )
            or []
        )



from enum import Enum


class KBDocumentStatusEnum(str, Enum):
    INACTIVE = "inactive"
    ACTIVE = "active"
    PREPROCESSING = "preprocessing"


class KBDocumentColumnDataTypeEnum(str, Enum):
    TEXT = "text"
    NUMBER = "number"
    BOOLEAN = "boolean"
    GEOLOCATION = "geolocation"
    IMAGE = "image"
    DATE = "date"
    ARRAY = "array"
    OBJECT = "object"


class KBDocumentColumnFilterTypeEnum(str, Enum):
    ENUM = "enum"
    RANGE = "range"
    TEXT = "text"



import datetime
import logging
from enum import Enum
from typing import Optional, Union

from dateutil import parser
from dateutil.tz import gettz

from filum_utils.errors import BaseError, ErrorMessage


class DateFormatOrder(str, Enum):
    YEAR_FIRST = "year_first"
    DAY_FIRST = "day_first"


class DateTimeFormatter:
    """A class to handle datetime formatting operations with Vietnam timezone"""

    VIETNAM_TIMEZONE = "Asia/Ho_Chi_Minh"

    @classmethod
    def get_vietnam_tz(cls) -> datetime.tzinfo:
        """Get Vietnam timezone object"""
        tz = gettz(cls.VIETNAM_TIMEZONE)
        if tz is None:
            raise BaseError(ErrorMessage.TIMEZONE_ERROR)
        return tz

    @classmethod
    def get_current_datetime(cls) -> datetime.datetime:
        """Get current datetime in Vietnam timezone"""
        return datetime.datetime.now(tz=cls.get_vietnam_tz())

    @classmethod
    def get_current_date(cls) -> datetime.date:
        """Get current date in Vietnam timezone"""
        return cls.get_current_datetime().date()

    @classmethod
    def get_current_datetime_str(cls) -> str:
        """Get current datetime as string"""
        return str(cls.get_current_datetime())

    @classmethod
    def timestamp_to_datetime(
        cls,
        timestamp: Union[int, float, str, None],
        is_milliseconds: bool = True
    ) -> Optional[datetime.datetime]:
        """Convert timestamp to datetime object

        Args:
            timestamp: Unix timestamp in milliseconds or seconds
            is_milliseconds: True if timestamp is in milliseconds, False if in seconds

        Returns:
            datetime object in Vietnam timezone or None if timestamp is empty
        """
        if not timestamp:
            return None

        timestamp_float = float(timestamp)
        timestamp_float = timestamp_float / 1000 if is_milliseconds else timestamp_float

        return datetime.datetime.fromtimestamp(
            timestamp_float,
            tz=cls.get_vietnam_tz()
        )

    @classmethod
    def timestamp_to_datetime_str(
        cls,
        timestamp: Union[int, float, str, None],
        is_milliseconds: bool = True
    ) -> Optional[str]:
        """Convert timestamp to datetime string"""
        formatted_datetime = cls.timestamp_to_datetime(timestamp, is_milliseconds)
        return str(formatted_datetime) if formatted_datetime else None

    @classmethod
    def format_to_dmy(cls, datetime_str: Optional[str]) -> Optional[str]:
        """Convert datetime/date string to dd/mm/yyyy format
        
        Args:
            datetime_str: Input datetime or date string
            
        Returns:
            Date string in dd/mm/yyyy format (e.g., "31/12/2023") or None if input is empty
            
        Raises:
            BaseError: If string cannot be formatted
        """
        if not datetime_str:
            return None

        parsed_datetime = cls.str_to_datetime(datetime_str)
        if not parsed_datetime:
            raise BaseError(
                message=ErrorMessage.INVALID_DATETIME_STRING,
                data={"datetime_str": datetime_str}
            )

        return parsed_datetime.strftime("%d/%m/%Y")

    @classmethod
    def get_days_ago_datetime(cls, days_ago: int) -> datetime.datetime:
        """Get datetime object for specified days ago"""
        return cls.get_current_datetime() - datetime.timedelta(days=days_ago)

    @classmethod
    def get_days_ago_datetime_str(cls, days_ago: int) -> str:
        """Get datetime string for specified days ago"""
        days_ago_datetime = cls.get_days_ago_datetime(days_ago)
        return str(days_ago_datetime)

    @classmethod
    def str_to_datetime(cls, datetime_str: str) -> Optional[datetime.datetime]:
        """Convert string to datetime object with automatic format detection
        
        Handles various datetime string formats and converts them to datetime objects
        with Vietnam timezone. Supports both date-first (dd-mm-yyyy) and 
        year-first (yyyy-mm-dd) formats.
        
        Args:
            datetime_str: Input datetime string (e.g., "2023-12-31 15:30:00" or "31-12-2023 15:30:00")
            
        Returns:
            datetime object in Vietnam timezone or None if parsing fails
        """
        if not datetime_str:
            return None

        normalized_datetime_str = cls._normalize_datetime_format(datetime_str)
        datetime_str_with_timezone = cls._add_timezone(normalized_datetime_str)

        date_part = datetime_str_with_timezone.split("T")[0]
        format_order = cls._detect_date_format(date_part)

        if not format_order:
            logging.error(f"Unable to detect date format in string: {datetime_str}")
            return None

        return cls._parse_to_datetime(
            datetime_str_with_timezone,
            is_day_first=(format_order == DateFormatOrder.DAY_FIRST)
        )

    @staticmethod
    def _normalize_datetime_format(datetime_str: str) -> str:
        """Standardize datetime string format
        
        Converts various datetime string formats to a standard format:
        - Replaces spaces with 'T' for ISO format
        - Converts forward slashes to hyphens for consistent parsing
        """
        return datetime_str.replace(" ", "T").replace("/", "-")

    @classmethod
    def _add_timezone(cls, datetime_str: str) -> str:
        """Add Vietnam timezone (+07:00) to datetime string if missing
        
        Handles various cases:
        - Strings ending with 'Z' (UTC)
        - Strings without timezone
        - Strings with existing timezone
        """
        if not datetime_str:
            return datetime_str

        # Handle UTC timezone 
        if datetime_str.endswith("Z"):
            datetime_str = datetime_str.replace("Z", "")
            return f"{datetime_str}+00:00"

        # Check if string contains time part
        parts = datetime_str.split("T")
        if len(parts) != 2:
            return datetime_str

        # Add timezone if missing
        time_part = parts[1]
        return datetime_str if "+" in time_part else f"{datetime_str}+07:00"

    @staticmethod
    def _detect_date_format(date_str: str) -> Optional[DateFormatOrder]:
        """Detect whether date string is in year-first or day-first format
        
        Attempts to parse the date string in both formats:
        - yyyy-mm-dd (year first)
        - dd-mm-yyyy (day first)
        
        Returns:
            DateFormatOrder enum indicating detected format or None if format is invalid
        """
        try:
            datetime.datetime.strptime(date_str, "%Y-%m-%d")
            return DateFormatOrder.YEAR_FIRST
        except ValueError:
            pass

        try:
            datetime.datetime.strptime(date_str, "%d-%m-%Y")
            return DateFormatOrder.DAY_FIRST
        except ValueError:
            return None

    @classmethod
    def _parse_to_datetime(
        cls,
        datetime_str: str,
        is_day_first: bool = False
    ) -> Optional[datetime.datetime]:
        """Parse datetime string using detected format
        
        Args:
            datetime_str: Normalized datetime string with timezone
            is_day_first: True if date is in dd-mm-yyyy format, False for yyyy-mm-dd
            
        Returns:
            datetime object in Vietnam timezone or None if parsing fails
        """
        if not datetime_str:
            return None

        return parser.parse(
            datetime_str,
            dayfirst=is_day_first,
            yearfirst=not is_day_first
        ).astimezone(tz=cls.get_vietnam_tz())



from typing import List, Optional

from tenacity import retry

from filum_utils.clients.contact_center.common import _RETRY_PARAMS, ContactCenterClient
from filum_utils.clients.contact_center.types import MessageAttachmentType


class ConversationClient(ContactCenterClient):
    @retry(**_RETRY_PARAMS)
    def update_origin_message_id(
        self,
        organization_id: str,
        conversation_id: str,
        message_id: str,
        original_message_id: Optional[str],
        status: Optional[str] = None,
        failed_reason: Optional[str] = None,
        failed_reason_type: Optional[str] = None,
        attachments: Optional[List[MessageAttachmentType]] = None,
    ):
        request_data = {}
        if original_message_id:
            request_data["messageId"] = original_message_id

        if status:
            request_data["status"] = status

        if failed_reason:
            request_data["failedReason"] = failed_reason

        if failed_reason_type:
            request_data["failedReasonType"] = failed_reason_type

        if attachments:
            request_data["attachments"] = attachments

        if not request_data:
            return

        self._request(
            method="PUT",
            endpoint=f"/internal/conversations/{conversation_id}/messages/{message_id}",
            data=request_data,
            params={
                "organizationId": organization_id,
            },
        )



import logging
from json import JSONDecodeError
from typing import Any, Dict, List, Union

import requests
from requests.auth import HTTPBasicAuth

from filum_utils.errors import ErrorMessage, RequestBaseError


def retry_if_error(exception):
    is_request_base_error = isinstance(exception, RequestBaseError)
    if is_request_base_error and exception.error_code in [503, 504]:
        return True

    return False


class BaseClient:
    def __init__(self, base_url: str, username: str = None, password: str = None):
        self._base_url = base_url
        self._username = username
        self._password = password

    def _request(
        self, method, endpoint, data=None, params=None, timeout: float = None
    ) -> Union[Dict[str, Any], List[Any], bytes]:
        response = requests.request(
            method=method,
            url=self._base_url + endpoint,
            json=data,
            params=params,
            auth=HTTPBasicAuth(self._username, self._password),
            timeout=timeout,
        )

        if response.status_code >= 300:
            logging.error(
                f"{self.__class__} error: {response.status_code} - {endpoint} - {response.content}"
            )
            raise RequestBaseError(
                ErrorMessage.IMPLEMENTATION_ERROR,
                error_code=response.status_code,
                data={
                    "method": method,
                    "endpoint": endpoint,
                    "params": params,
                    "status_code": response.status_code,
                    "content": str(response.content),
                },
            )
        try:
            return response.json()
        except JSONDecodeError:
            return response.content



from typing import Any, Dict, List, Optional, TypedDict

from filum_utils.clients.knowledge_base.enums import (
    KBDocumentColumnDataTypeEnum,
    KBDocumentColumnFilterTypeEnum,
    KBDocumentStatusEnum,
)


class UpsertDocumentChunkDataType(TypedDict):
    text: str
    categories: List[str]
    summary: str


class UpsertDocumentColumnSchemaDataType(TypedDict, total=False):
    column_name: str
    data_type: KBDocumentColumnDataTypeEnum
    filter_type: Optional[KBDocumentColumnFilterTypeEnum]
    is_primary_key: bool


class UpsertDocumentDataType(TypedDict, total=False):
    document_id: str
    status: KBDocumentStatusEnum
    table_records: List[Dict[str, Any]]
    column_schemas: List[UpsertDocumentColumnSchemaDataType]
    collection_name: str


class PreprocessUpsertDocumentRequestType(TypedDict, total=False):
    document_id: str
    object_key: str
    status: KBDocumentStatusEnum
    sheet_name: Optional[str]



from datetime import date, datetime
from typing import Dict, List, Optional

from tenacity import retry, retry_if_exception, stop_after_attempt, wait_fixed

from filum_utils.clients.common import BaseClient, retry_if_error
from filum_utils.config import config
from filum_utils.enums import Organization
from filum_utils.types.engagement_message import (
    CreateEngagementMessage,
    UpdateEngagementMessageType,
)
from filum_utils.types.survey_response import CreateSurveyResponse
from filum_utils.utils.datetime_formatter import DateTimeFormatter

_RETRY_PARAMS = {
    "reraise": True,
    "wait": wait_fixed(10),
    "stop": stop_after_attempt(3),
    "retry": retry_if_exception(retry_if_error),
}


class FilumClient(BaseClient):
    def __init__(self):
        super().__init__(
            base_url=config.FILUM_BASE_URL,
            username=config.FILUM_USERNAME,
            password=config.FILUM_PASSWORD,
        )

    @retry(**_RETRY_PARAMS)
    def get_automated_action(self, automated_action_id: int):
        return self._request(
            method="GET", endpoint=f"/internal/automated-actions/{automated_action_id}"
        )

    @retry(**_RETRY_PARAMS)
    def get_campaign(self, campaign_id: str):
        return self._request(
            method="GET", endpoint=f"/internal/survey-campaigns/{campaign_id}"
        )

    @retry(**_RETRY_PARAMS)
    def get_segment(self, segment_id: str, organization: Organization):
        return self._request(
            method="GET",
            endpoint=f"/internal/segments/{segment_id}",
            params={
                "organization_id": organization["id"],
                "organization_slug": organization["slug"],
            },
        )

    @retry(**_RETRY_PARAMS)
    def update_campaign_subscription_status(
        self, campaign_id: str, subscription_id: str, updated_status: str
    ):
        return self._request(
            method="PUT",
            endpoint=f"/internal/survey-campaigns/{campaign_id}/subscription-webhooks",
            data={"subscription_id": subscription_id, "status": updated_status},
        )

    @retry(**_RETRY_PARAMS)
    def update_engagement_campaign_subscription_status(
        self,
        organization_id: str,
        campaign_id: str,
        distribution_id: str,
        updated_status: str,
    ):
        return self._request(
            method="PUT",
            endpoint=f"/internal/engagement-campaigns/{campaign_id}/distributions/{distribution_id}/status",
            params={"organization_id": organization_id},
            data={"status": updated_status},
        )

    @retry(**_RETRY_PARAMS)
    def get_user_csv_reader(
        self,
        custom_properties: List[str],
        segment_id: str,
        organization: Organization,
        limit: int = config.SEGMENT_RECORD_LIMIT,
        offset: int = 0,
        user_properties: Optional[List[str]] = None,
    ):
        return self._request(
            method="POST",
            endpoint="/internal/users/export",
            data={
                "organization_id": organization["id"],
                "organization_slug": organization["slug"],
                "segment_id": segment_id,
                "custom_properties": custom_properties,
                "user_properties": user_properties,
                "offset": offset,
                "limit": limit,
            },
            timeout=300,
        )

    @retry(**_RETRY_PARAMS)
    def get_uploaded_file(self, file_name):
        return self._request(
            method="POST",
            endpoint="/internal/download-file",
            data={
                "file_name": file_name,
            },
        )

    @retry(**_RETRY_PARAMS)
    def create_survey_responses(
        self, campaign_id: str, responses: List[CreateSurveyResponse]
    ):
        return self._request(
            method="POST",
            endpoint=f"/internal/survey-campaigns/{campaign_id}/survey-responses/batch",
            data=responses,
        )

    @retry(**_RETRY_PARAMS)
    def create_engagement_messages(
        self,
        organization_id: str,
        campaign_id: str,
        messages: List[CreateEngagementMessage],
    ):
        return self._request(
            method="POST",
            endpoint=f"/internal/engagement-campaigns/{campaign_id}/messages/batch",
            params={"organization_id": organization_id},
            data=messages,
        )

    @retry(**_RETRY_PARAMS)
    def get_sent_engagement_message_count(
        self, organization_id: str, campaign_id: str, day: date
    ):
        return self._request(
            method="GET",
            endpoint=f"/internal/engagement-campaigns/{campaign_id}/sent-message-count",
            params={
                "organization_id": organization_id,
                "day": day.isoformat(),
            },
        )

    @retry(**_RETRY_PARAMS)
    def get_this_month_answered_response_counts_by_metadata(
        self, organization_id: str, campaign_id: str, metadata_id: str, source: str
    ) -> Dict[str, int]:
        """Retrieve total survey responses for each metadata value for current month.

        Args:
            organization_id (str): ID of the organization
            campaign_id (str): ID of the campaign
            metadata_id (str): Metadata ID
            source (str)

        Returns:
            Dict[str, int]: Mapping from metadata value to survey response count
        """
        # Get the first date of the current month as start_date
        current_datetime = DateTimeFormatter.get_current_datetime()
        start_date = current_datetime.replace(day=1).date()
        # Get the current date as end_date
        end_date = current_datetime.date()

        return self._request(
            method="GET",
            endpoint=f"/internal/survey-campaigns/{campaign_id}/answered-response-counts",
            params={
                "organization_id": organization_id,
                "metadata_id": metadata_id,
                "start_date": str(start_date),
                "end_date": str(end_date),
                "source": source,
            },
        )

    @retry(**_RETRY_PARAMS)
    def update_survey_response(
        self,
        identifier: str,
        sent_timestamp: Optional[datetime] = None,
        sending_failed_reason: Optional[str] = None,
        original_message_id: Optional[str] = None,
        send_cost: Optional[float] = None,
    ):
        self._request(
            method="PUT",
            endpoint=f"/internal/survey-responses/{identifier}",
            data={
                "send_timestamp": str(sent_timestamp),
                "sending_failed_reason": sending_failed_reason,
                "original_message_id": original_message_id,
                "send_cost": send_cost,
            },
        )

    @retry(**_RETRY_PARAMS)
    def update_engagement_message(
        self, organization_id: str, engagement_message: UpdateEngagementMessageType
    ):
        message_id = engagement_message["id"]
        self._request(
            method="PUT",
            endpoint=f"/internal/engagement-messages/{message_id}",
            params={"organization_id": organization_id},
            data=engagement_message,
        )



from .clients.action import ActionClient
from .clients.analytics import AnalyticsClient
from .clients.connection import ConnectionClient
from .clients.contact_center.conversation import ConversationClient
from .clients.contact_center.document import DocumentClient
from .clients.iam import IAMClient
from .clients.knowledge_base.client import KnowledgeBaseClient
from .clients.mini_app import InstalledMiniAppClient
from .clients.subscription import SubscriptionClient
from .services.subscription.automated_action import AutomatedActionSubscriptionService
from .services.subscription.campaign import CampaignSubscriptionService
from .utils.automated_action.string_formatter import AutomatedActionStringFormatter
from .utils.datetime_formatter import DateTimeFormatter
from .utils.engagement_campaign.string_formatter import ContentTemplate, ZaloTemplate
from .utils.string_formatter import StringFormatter

__all__ = [
    ActionClient,
    AnalyticsClient,
    AutomatedActionSubscriptionService,
    AutomatedActionStringFormatter,
    CampaignSubscriptionService,
    ContentTemplate,
    ConnectionClient,
    ConversationClient,
    DateTimeFormatter,
    DocumentClient,
    IAMClient,
    InstalledMiniAppClient,
    SubscriptionClient,
    StringFormatter,
    ZaloTemplate,
    KnowledgeBaseClient,
]



from typing import List, Optional

from tenacity import retry

from filum_utils.clients.contact_center.common import _RETRY_PARAMS, ContactCenterClient
from filum_utils.clients.contact_center.types import (
    KBDocumentType,
    UpdateDocumentEmbeddingType,
)


class DocumentClient(ContactCenterClient):
    @retry(**_RETRY_PARAMS)
    def get_kb_document(self, organization_id: str, document_id: str) -> KBDocumentType:
        return self._request(
            endpoint=f"/internal/documents/{document_id}",
            method="GET",
            params={"organizationId": organization_id},
        )

    @retry(**_RETRY_PARAMS)
    def update_document_embedding_batch(
        self,
        organization_id: str,
        document_embeddings: List[UpdateDocumentEmbeddingType],
        batch_id: Optional[str] = None,
        batch_status: Optional[str] = None,
    ):
        request_body = {"batch_progress": 100, "documents": document_embeddings}
        if batch_id:
            request_body["batch_id"] = batch_id

        if batch_status:
            request_body["batch_status"] = batch_status

        self._request(
            endpoint="/internal/documents/embedding",
            method="PUT",
            params={"organizationId": organization_id},
            data=request_body,
        )



from enum import Enum


# Conversation Enums
class MessageSenderTypeEnum(str, Enum):
    USER = "user"
    AGENT = "agent"
    SYSTEM = "system"
    AI = "ai"


class ContactCenterMessageTypeEnum(str, Enum):
    CHAT = "chat"
    COMMENT = "comment"


class MessageStatusEnum(str, Enum):
    SENT = "sent"
    FAILED = "failed"
    DELIVERED = "delivered"
    SEEN = "seen"


class PubSubMessageEventNameEnum(str, Enum):
    MESSAGE_CREATED = "Message Created"
    MESSAGE_STATUS_UPDATED = "Message Status Updated"
    CALL_CREATED = "Call Created"


# Document Enums
class DocumentStatusEnums(str, Enum):
    DRAFT = "draft"
    PREPROCESSING = "preprocessing"
    PREPROCESSED = "preprocessed"
    EMBEDDING = "embedding"
    FAILED = "failed"
    INACTIVE = "inactive"
    ACTIVE = "active"
    DELETING = "deleting"
    DELETED = "deleted"


class DocumentSyncStatusEnum(str, Enum):
    PENDING = "pending"
    PREPROCESSING = "preprocessing"
    PROCESSING = "processing"
    EMBEDDING = "embedding"
    FAILED = "failed"
    SUCCESS = "success"


class DocumentDataFormatEnum(str, Enum):
    TABLE = "table"
    VECTOR = "vector"


class DocumentColumnDataTypeEnum(str, Enum):
    TEXT = "text"
    NUMBER = "number"
    BOOLEAN = "boolean"
    GEOLOCATION = "geolocation"
    IMAGE = "image"
    DATE = "date"
    ARRAY = "array"
    OBJECT = "object"


class DocumentColumnFilterTypeEnum(str, Enum):
    ENUM = "enum"
    RANGE = "range"
    TEXT = "text"
